{% extends "../change_form.html" %}


{% block djstripewarning %}
    <p class="stripe-danger-warning">Any Changes made here will also be updated on your Stripe Account. Proceed with Caution.</p>
{% endblock djstripewarning %}
